@@ -38,8 +38,6 @@ struct _CcShellCategoryViewPrivate
 {
   gchar *name;
   GtkTreeModel *model;
-
-  GtkWidget *header;
   GtkWidget *iconview;
 };
 
@@ -116,12 +114,34 @@ cc_shell_category_view_finalize (GObject *object)
   G_OBJECT_CLASS (cc_shell_category_view_parent_class)->finalize (object);
 }
 
+static void
+label_style_set_cb (GtkWidget *widget,
+                    GtkStyle  *old_style,
+                    gpointer   user_data)
+{
+  GtkStyle *style;
+
+  /* "base" colours are used for the background inside CcShellCategoryView,
+   * so set the labels to use the "text" colors */
+
+  g_signal_handlers_block_by_func (widget, label_style_set_cb, NULL);
+
+  style = gtk_widget_get_style (widget);
+
+  gtk_widget_modify_fg (widget, GTK_STATE_NORMAL,
+                        &style->text[GTK_STATE_NORMAL]);
+
+  g_signal_handlers_unblock_by_func (widget, label_style_set_cb, NULL);
+}
+
+
+
 static void
 cc_shell_category_view_constructed (GObject *object)
 {
   CcShellCategoryViewPrivate *priv = CC_SHELL_CATEGORY_VIEW (object)->priv;
-  gchar *header_name;
-  GtkWidget *iconview, *vbox, *header, *self;
+  GtkWidget *iconview, *vbox, *self;
+  GtkWidget *alignment;
 
   self = GTK_WIDGET (object);
 
@@ -132,59 +152,59 @@ cc_shell_category_view_constructed (GObject *object)
 
   gtk_icon_view_set_pixbuf_column (GTK_ICON_VIEW (iconview), COL_PIXBUF);
   gtk_icon_view_set_text_column (GTK_ICON_VIEW (iconview), COL_NAME);
-  gtk_icon_view_set_item_width (GTK_ICON_VIEW (iconview), 120);
+  gtk_icon_view_set_item_width (GTK_ICON_VIEW (iconview), 100);
 
   /* create the header if required */
   if (priv->name)
     {
-      header_name = g_strdup_printf ("<b>%s</b>", priv->name);
+      GtkWidget *label;
+      PangoAttrList *attrs;
 
-      header = g_object_new (GTK_TYPE_LABEL,
-                             "use-markup", TRUE,
-                             "label", header_name,
-                             "wrap", TRUE,
-                             "xalign", 0.0,
-                             "xpad", 6,
-                             NULL);
+      label = gtk_label_new (priv->name);
+      attrs = pango_attr_list_new ();
+      pango_attr_list_insert (attrs, pango_attr_weight_new (PANGO_WEIGHT_BOLD));
+      gtk_label_set_attributes (GTK_LABEL (label), attrs);
+      pango_attr_list_unref (attrs);
+      gtk_frame_set_label_widget (GTK_FRAME (object), label);
 
-      g_free (header_name);
-      gtk_box_pack_start (GTK_BOX (vbox), header, FALSE, TRUE, 3);
-
-      priv->header = header;
+      g_signal_connect (label, "style-set", G_CALLBACK (label_style_set_cb),
+                        NULL);
     }
 
   /* add the iconview to the vbox */
   gtk_box_pack_start (GTK_BOX (vbox), iconview, FALSE, TRUE, 0);
 
   /* add the main vbox to the view */
-  gtk_container_add (GTK_CONTAINER (object), vbox);
-  gtk_widget_show_all (vbox);
+  alignment = gtk_alignment_new (0, 0, 1, 1);
+  gtk_alignment_set_padding (GTK_ALIGNMENT (alignment), 0, 0, 12, 0);
+  gtk_container_add (GTK_CONTAINER (alignment), vbox);
+  gtk_container_add (GTK_CONTAINER (object), alignment);
+  gtk_widget_show_all (alignment);
 
   priv->iconview = iconview;
 }
 
-static void
-cc_shell_category_view_style_set (GtkWidget *widget,
-                                  GtkStyle  *old_style)
+static gboolean
+cc_shell_category_view_draw (GtkWidget *widget,
+                             cairo_t   *cr)
 {
-  CcShellCategoryViewPrivate *priv = CC_SHELL_CATEGORY_VIEW (widget)->priv;
-  GtkStyle *style = gtk_widget_get_style (widget);
+  GtkStyle *style;
+  GtkStateType state;
+  GtkAllocation allocation;
 
-  if (priv->header)
-    {
-      gtk_widget_modify_bg (priv->header, GTK_STATE_NORMAL,
-                            &style->base[GTK_STATE_NORMAL]);
-      gtk_widget_modify_fg (priv->header, GTK_STATE_NORMAL,
-                            &style->text[GTK_STATE_NORMAL]);
-    }
+  style = gtk_widget_get_style (widget);
+  state = gtk_widget_get_state (widget);
+  gtk_widget_get_allocation (widget, &allocation);
 
-  if (priv->iconview)
-    {
-      gtk_widget_modify_bg (priv->iconview, GTK_STATE_NORMAL,
-                            &style->base[GTK_STATE_NORMAL]);
-      gtk_widget_modify_fg (priv->iconview, GTK_STATE_NORMAL,
-                            &style->text[GTK_STATE_NORMAL]);
-    }
+
+  cairo_rectangle (cr, 0, 0, allocation.width, allocation.height);
+  gdk_cairo_set_source_color (cr, &style->base[state]);
+
+  cairo_fill (cr);
+
+  GTK_WIDGET_CLASS (cc_shell_category_view_parent_class)->draw (widget, cr);
+
+  return FALSE;
 }
 
 static void
@@ -202,7 +222,7 @@ cc_shell_category_view_class_init (CcShellCategoryViewClass *klass)
   object_class->finalize = cc_shell_category_view_finalize;
   object_class->constructed = cc_shell_category_view_constructed;
 
-  widget_class->style_set = cc_shell_category_view_style_set;
+  widget_class->draw = cc_shell_category_view_draw;
 
   pspec = g_param_spec_string ("name",
                                "Name",
