@@ -23,6 +23,8 @@
 #include <string.h>
 
 #define GNOME_SETTINGS_PANEL_ID_KEY "X-GNOME-Settings-Panel"
+#define GNOME_SETTINGS_PANEL_CATEGORY GNOME_SETTINGS_PANEL_ID_KEY
+#define GNOME_SETTINGS_PANEL_ID_KEYWORDS "X-GNOME-Keywords"
 
 
 G_DEFINE_TYPE (CcShellModel, cc_shell_model, GTK_TYPE_LIST_STORE)
@@ -37,7 +39,7 @@ static void
 cc_shell_model_init (CcShellModel *self)
 {
   GType types[] = {G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING,
-      GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING};
+      GDK_TYPE_PIXBUF, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRV};
 
   gtk_list_store_set_column_types (GTK_LIST_STORE (self),
                                    N_COLS, types);
@@ -49,6 +51,36 @@ cc_shell_model_new (void)
   return g_object_new (CC_TYPE_SHELL_MODEL, NULL);
 }
 
+static gboolean
+desktop_entry_has_panel_category (GKeyFile *key_file)
+{
+  char   **strv;
+  gsize    len;
+  int      i;
+
+  strv = g_key_file_get_string_list (key_file,
+				     "Desktop Entry",
+				     "Categories",
+				     &len,
+				     NULL);
+  if (!strv)
+    return FALSE;
+
+  for (i = 0; strv[i]; i++)
+    {
+      if (g_str_equal (strv[i], GNOME_SETTINGS_PANEL_CATEGORY))
+        {
+          g_strfreev (strv);
+          return TRUE;
+	}
+    }
+
+  g_strfreev (strv);
+
+  return FALSE;
+
+}
+
 void
 cc_shell_model_add_item (CcShellModel   *model,
                          const gchar    *category_name,
@@ -64,6 +96,7 @@ cc_shell_model_add_item (CcShellModel   *model,
   GError *err = NULL;
   gchar *search_target;
   GKeyFile *key_file;
+  gchar **keywords;
 
   /* load the .desktop file since gnome-menus doesn't have a way to read
    * custom properties from desktop files */
@@ -73,11 +106,29 @@ cc_shell_model_add_item (CcShellModel   *model,
 
   id = g_key_file_get_string (key_file, "Desktop Entry",
                               GNOME_SETTINGS_PANEL_ID_KEY, NULL);
-  g_key_file_free (key_file);
-  key_file = NULL;
 
   if (!id)
-    id = g_strdup (gmenu_tree_entry_get_desktop_file_id (item));
+    {
+      /* Refuse to load desktop files without a panel ID, but
+       * with the X-GNOME-Control-Center-Panel category */
+      if (desktop_entry_has_panel_category (key_file))
+        {
+          g_warning ("Not loading desktop file '%s' because it uses the "
+		     GNOME_SETTINGS_PANEL_CATEGORY
+		     " category but isn't a panel.",
+		     desktop);
+         g_key_file_free (key_file);
+         return;
+	}
+      id = g_strdup (gmenu_tree_entry_get_desktop_file_id (item));
+    }
+
+  keywords = g_key_file_get_locale_string_list (key_file, "Desktop Entry",
+                                                GNOME_SETTINGS_PANEL_ID_KEYWORDS,
+                                                NULL, NULL, NULL);
+
+  g_key_file_free (key_file);
+  key_file = NULL;
 
   /* find the icon */
   if (icon != NULL && *icon == '/')
@@ -114,8 +165,10 @@ cc_shell_model_add_item (CcShellModel   *model,
                                      COL_CATEGORY, category_name,
                                      COL_SEARCH_TARGET, search_target,
                                      COL_ICON_NAME, icon,
+                                     COL_KEYWORDS, keywords,
                                      -1);
 
   g_free (id);
   g_free (search_target);
+  g_strfreev (keywords);
 }
