@@ -56,6 +56,8 @@ struct _GnomeControlCenterPrivate
 {
   GtkBuilder *builder;
   GtkWidget  *notebook;
+  GtkWidget  *main_vbox;
+  GtkWidget  *scrolled_window;
   GtkWidget  *window;
   GtkWidget  *search_entry;
 
@@ -72,8 +74,6 @@ struct _GnomeControlCenterPrivate
 
   gchar *default_window_title;
   gchar *default_window_icon;
-
-  gint overview_height;
 };
 
 /* Use a fixed width for the shell, since resizing horizontally is more awkward
@@ -125,8 +125,8 @@ activate_panel (GnomeControlCenter *shell,
         {
           GtkWidget *panel;
           GtkWidget *box;
-          gint i, old_page;
-          GtkRequisition min, nat;
+          gint i;
+          int nat_height;
 
           /* create the panel plugin */
           panel = g_object_new (panel_type, "shell", shell, NULL);
@@ -136,17 +136,6 @@ activate_panel (GnomeControlCenter *shell,
 
           gtk_container_add (GTK_CONTAINER (box), panel);
 
-          old_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (priv->notebook));
-
-          /* store the old size */
-          if (old_page == OVERVIEW_PAGE)
-            {
-              GtkAllocation alloc;
-
-              gtk_widget_get_allocation (priv->window, &alloc);
-              priv->overview_height = alloc.height;
-            }
-
           i = gtk_notebook_append_page (GTK_NOTEBOOK (priv->notebook), box,
                                         NULL);
 
@@ -160,13 +149,18 @@ activate_panel (GnomeControlCenter *shell,
 
           gtk_widget_show (panel);
 
+          /* set the scrolled window small so that it doesn't force
+             the window to be larger than this panel */
+          gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (priv->scrolled_window), 50);
+
           /* resize to the preferred size of the panel */
           gtk_widget_set_size_request (priv->window, FIXED_WIDTH, -1);
-
-          gtk_widget_get_preferred_size (priv->window,
-                                         &min, &nat);
-          gtk_window_resize (GTK_WINDOW (priv->window), FIXED_WIDTH,
-                             nat.height);
+          gtk_widget_get_preferred_height (GTK_WIDGET (priv->window),
+                                           NULL, &nat_height);
+          g_debug ("Setting panel height: %d", nat_height);
+          gtk_window_resize (GTK_WINDOW (priv->window),
+                             FIXED_WIDTH,
+                             nat_height);
           return;
         }
     }
@@ -217,7 +211,6 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
 
   gtk_notebook_remove_page (GTK_NOTEBOOK (priv->notebook), CAPPLET_PAGE);
 
-
   /* clear the search text */
   g_free (priv->filter_string);
   priv->filter_string = g_strdup ("");
@@ -227,21 +220,8 @@ shell_show_overview_page (GnomeControlCenterPrivate *priv)
   gtk_window_set_title (GTK_WINDOW (priv->window), priv->default_window_title);
   gtk_window_set_icon_name (GTK_WINDOW (priv->window),
                             priv->default_window_icon);
-
-  /* resize back to the original overview height */
-  if (priv->overview_height > 0)
-    {
-      gtk_widget_set_size_request (priv->window,
-                                   FIXED_WIDTH,
-                                   priv->overview_height);
-      gtk_window_resize (GTK_WINDOW (priv->window),
-                         FIXED_WIDTH,
-                         priv->overview_height);
-    }
 }
 
-
-
 static void
 item_activated_cb (CcShellCategoryView *view,
                    gchar               *name,
@@ -294,14 +274,10 @@ category_focus_in (GtkWidget          *view,
 static GList *
 get_item_views (GnomeControlCenter *shell)
 {
-  GnomeControlCenterPrivate *priv = shell->priv;
-  GtkWidget *vbox;
   GList *list, *l;
   GList *res;
 
-  vbox = W (priv->builder, "main-vbox");
-
-  list = gtk_container_get_children (GTK_CONTAINER (vbox));
+  list = gtk_container_get_children (GTK_CONTAINER (shell->priv->main_vbox));
   res = NULL;
   for (l = list; l; l = l->next)
     {
@@ -424,14 +400,18 @@ model_filter_func (GtkTreeModel              *model,
   gchar *name, *target;
   gchar *needle, *haystack;
   gboolean result;
+  gchar **keywords;
 
   gtk_tree_model_get (model, iter, COL_NAME, &name,
-                      COL_SEARCH_TARGET, &target, -1);
+                      COL_SEARCH_TARGET, &target,
+                      COL_KEYWORDS, &keywords,
+                      -1);
 
   if (!priv->filter_string || !name || !target)
     {
       g_free (name);
       g_free (target);
+      g_strfreev (keywords);
       return FALSE;
     }
 
@@ -440,10 +420,24 @@ model_filter_func (GtkTreeModel              *model,
 
   result = (strstr (haystack, needle) != NULL);
 
+  if (!result && keywords)
+    {
+      gint i;
+      gchar *keyword;
+
+      for (i = 0; !result && keywords[i]; i++)
+        {
+          keyword = g_utf8_casefold (keywords[i], -1);
+          result = strstr (keyword, needle) == keyword;
+          g_free (keyword);
+        }
+    }
+
   g_free (name);
   g_free (target);
   g_free (haystack);
   g_free (needle);
+  g_strfreev (keywords);
 
   return result;
 }
@@ -602,15 +596,12 @@ fill_model (GnomeControlCenter *shell)
   GSList *list, *l;
   GMenuTreeDirectory *d;
   GMenuTree *tree;
-  GtkWidget *vbox;
-  GtkWidget *sw;
 
   GnomeControlCenterPrivate *priv = shell->priv;
 
-  vbox = W (priv->builder, "main-vbox");
-  sw = W (priv->builder, "scrolledwindow1");
-  gtk_container_set_focus_vadjustment (GTK_CONTAINER (vbox),
-                                       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (sw)));
+  gtk_container_set_border_width (GTK_CONTAINER (shell->priv->main_vbox), 10);
+  gtk_container_set_focus_vadjustment (GTK_CONTAINER (shell->priv->main_vbox),
+                                       gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (shell->priv->scrolled_window)));
 
   tree = gmenu_tree_lookup (MENUDIR "/gnomecc.menu", 0);
 
@@ -649,7 +640,7 @@ fill_model (GnomeControlCenter *shell)
                                                   g_strdup (dir_name), g_free);
 
           categoryview = cc_shell_category_view_new (dir_name, filter);
-          gtk_box_pack_start (GTK_BOX (vbox), categoryview, FALSE, TRUE, 6);
+          gtk_box_pack_start (GTK_BOX (shell->priv->main_vbox), categoryview, FALSE, TRUE, 0);
           g_signal_connect (cc_shell_category_view_get_item_view (CC_SHELL_CATEGORY_VIEW (categoryview)),
                             "desktop-item-activated",
                             G_CALLBACK (item_activated_cb), shell);
@@ -801,8 +792,11 @@ _shell_set_active_panel_from_id (CcShell      *shell,
     }
 }
 
-
-
+static GtkWidget *
+_shell_get_toplevel (CcShell *shell)
+{
+  return GNOME_CONTROL_CENTER (shell)->priv->window;
+}
 
 /* GObject Implementation */
 static void
@@ -875,7 +869,6 @@ gnome_control_center_finalize (GObject *object)
 {
   GnomeControlCenterPrivate *priv = GNOME_CONTROL_CENTER (object)->priv;
 
-
   if (priv->filter_string)
     {
       g_free (priv->filter_string);
@@ -911,29 +904,68 @@ gnome_control_center_class_init (GnomeControlCenterClass *klass)
   object_class->finalize = gnome_control_center_finalize;
 
   shell_class->set_active_panel_from_id = _shell_set_active_panel_from_id;
+  shell_class->get_toplevel = _shell_get_toplevel;
 }
 
 static void
-vbox_style_set_cb (GtkWidget *widget,
-                   GtkStyle  *old_style)
+on_window_size_allocate (GtkWidget          *widget,
+                         GtkAllocation      *allocation,
+                         GnomeControlCenter *self)
+{
+  int height;
+
+  if (gtk_notebook_get_current_page (GTK_NOTEBOOK (self->priv->notebook)) == OVERVIEW_PAGE)
+    {
+      gtk_widget_get_preferred_height_for_width (GTK_WIDGET (self->priv->main_vbox),
+                                                 FIXED_WIDTH,
+                                                 NULL,
+                                                 &height);
+      if (gtk_widget_get_realized (widget))
+        {
+          int          monitor;
+          GdkScreen   *screen;
+          GdkRectangle rect;
+          GdkWindow   *window;
+
+          window = gtk_widget_get_window (widget);
+          screen = gtk_widget_get_screen (widget);
+          monitor = gdk_screen_get_monitor_at_window (screen, window);
+          gdk_screen_get_monitor_geometry (screen, monitor, &rect);
+          height = MIN (height + 10, rect.height - 120);
+        }
+    }
+  else
+    {
+      height = 50;
+    }
+
+  gtk_scrolled_window_set_min_content_height (GTK_SCROLLED_WINDOW (self->priv->scrolled_window), height);
+}
+
+static void
+viewport_style_set_cb (GtkWidget *widget,
+                       GtkStyle  *old_style,
+                       gpointer   user_data)
 {
-  GtkWidget *parent;
   GtkStyle *style;
 
-  parent = gtk_widget_get_parent (widget);
-  style = gtk_widget_get_style (widget);
-  gtk_widget_modify_bg (parent, GTK_STATE_NORMAL,
-                        &style->base[GTK_STATE_NORMAL]);
-  gtk_widget_modify_fg (parent, GTK_STATE_NORMAL,
-                        &style->text[GTK_STATE_NORMAL]);
+  /* use "base" colours inside the viewport */
 
+  g_signal_handlers_block_by_func (widget, viewport_style_set_cb, NULL);
+
+  style = gtk_widget_get_style (widget);
+
+  gtk_widget_modify_bg (widget, GTK_STATE_NORMAL,
+                        &style->base[GTK_STATE_NORMAL]);
+
+  g_signal_handlers_unblock_by_func (widget, viewport_style_set_cb, NULL);
 }
 
 static void
 gnome_control_center_init (GnomeControlCenter *self)
 {
+  GtkWidget *widget;
   GError *err = NULL;
-  GtkWidget *vbox;
   GnomeControlCenterPrivate *priv;
 
   priv = self->priv = CONTROL_CENTER_PRIVATE (self);
@@ -951,28 +983,24 @@ gnome_control_center_init (GnomeControlCenter *self)
 
   /* connect various signals */
   priv->window = W (priv->builder, "main-window");
-  g_signal_connect (priv->window, "delete-event", G_CALLBACK (gtk_main_quit),
-                    NULL);
+  g_signal_connect_swapped (priv->window, "delete-event", G_CALLBACK (g_object_unref), self);
 
   priv->notebook = W (priv->builder, "notebook");
+  priv->scrolled_window = W (priv->builder, "scrolledwindow1");
 
+  widget = W (priv->builder, "viewport");
+  g_signal_connect (widget, "style-set", G_CALLBACK (viewport_style_set_cb),
+                    NULL);
+
+  gtk_widget_set_size_request (priv->scrolled_window, FIXED_WIDTH, -1);
+  priv->main_vbox = W (priv->builder, "main-vbox");
+  g_signal_connect (priv->scrolled_window, "size-allocate", G_CALLBACK (on_window_size_allocate), self);
   g_signal_connect (priv->notebook, "switch-page",
                     G_CALLBACK (notebook_switch_page_cb), priv);
 
   g_signal_connect (gtk_builder_get_object (priv->builder, "home-button"),
                     "clicked", G_CALLBACK (home_button_clicked_cb), self);
 
-
-  /* setup background colours */
-  vbox = W (priv->builder, "main-vbox");
-  gtk_widget_set_size_request (vbox, 0, -1);
-
-  /* make sure the background of the vbox uses base/text colour combinations
-   * and is updated if the style changes */
-  vbox_style_set_cb (vbox, NULL);
-  g_signal_connect (vbox, "style-set", G_CALLBACK (vbox_style_set_cb),
-                    NULL);
-
   /* load the available settings panels */
   fill_model (self);
 
@@ -982,9 +1010,6 @@ gnome_control_center_init (GnomeControlCenter *self)
   /* setup search functionality */
   setup_search (self);
 
-
-  gtk_widget_show_all (priv->window);
-
   /* store default window title and name */
   priv->default_window_title = g_strdup (gtk_window_get_title (GTK_WINDOW (priv->window)));
   priv->default_window_icon = g_strdup (gtk_window_get_icon_name (GTK_WINDOW (priv->window)));
@@ -1001,3 +1026,11 @@ gnome_control_center_present (GnomeControlCenter *center)
 {
   gtk_window_present (GTK_WINDOW (center->priv->window));
 }
+
+void
+gnome_control_center_show (GnomeControlCenter *center,
+			   GtkApplication     *app)
+{
+  gtk_window_set_application (GTK_WINDOW (center->priv->window), app);
+  gtk_widget_show_all (gtk_bin_get_child (GTK_BIN (center->priv->window)));
+}
